from threading import Thread

from . import test_cipher
from . import test_digest


class ThreadedTester(Thread):
	failed = False

	def run(self):
		try:
			test_digest.test_digest()
			test_cipher.test_cipher(['a' * 1000, 'd' * 1000])
			# test_evp.test_rand()
		except Exception as e:
			self.failed = True
			self.exception = e


def test_threaded_crypto():
	threads = [ThreadedTester() for i in range(10)]
	map(lambda t: t.start(), threads)
	# wait for the threads to complete
	map(lambda t: t.join(), threads)
	assert all(not t.failed for t in threads), "Some threads failed"


if __name__ == '__main__':
	test_threaded_crypto()



import itertools

import pytest

from jaraco.crypto import cipher


def algorithm_modes():
	pairs = itertools.product(
		cipher.CIPHER_ALGORITHMS,
		cipher.CIPHER_MODES)
	for algorithm, mode in pairs:
		# apparently DES-EDE3-ECB is not a valid mode
		if (algorithm, mode) == ('DES-EDE3', 'ECB'):
			continue
		yield algorithm, mode


@pytest.mark.parametrize(
	['algorithm', 'mode'],
	algorithm_modes()
)
def test_cipher_type(algorithm, mode):
	# One can pass the algorithm and mode separately or together
	cipher.CipherType.from_name(algorithm, mode)
	cipher.CipherType.from_name(algorithm + '-' + mode)


@pytest.mark.parametrize('data_parts', [
	('a' * i, 'b' * i, 'c' * i) for i in range(0, 1000, 50)])
def test_cipher(data_parts):
	"""
	Encrypt and decrypt the data_parts supplied and ensure the source
	matches the result.
	"""
	key = '11111111111111111111111111111111'
	iv = '1111111111111111'
	params = ('AES-256', 'CBC'), key, iv
	ce = cipher.Cipher(*params)
	list(map(ce.update, data_parts))
	data_enc = ce.finalize()
	cd = cipher.Cipher(*params, encrypt=False)
	assert cd.finalize(data_enc) == ''.join(data_parts).encode()



import ctypes

import six

from . import evp


class DigestError(Exception):
	pass


class DigestType(ctypes.Structure):
	_fields_ = evp._digest_type_fields

	@classmethod
	def from_name(cls, digest_name):
		res = evp.get_digestbyname(digest_name.encode('ascii'))
		if not res:
			raise DigestError("Unknown Digest: %(digest_name)s" % vars())
		return res.contents


class Digest(ctypes.Structure):
	_fields_ = evp._digest_context_fields
	finalized = False

	def __init__(self, digest_type):
		self.digest_type = digest_type
		evp.MD_CTX_init(self)
		result = evp.DigestInit_ex(self, digest_type, None)
		if result == 0:
			raise DigestError("Unable to initialize digest")

	def update(self, data):
		if self.finalized:
			raise DigestError("Digest is finalized; no updates allowed")
		if isinstance(data, six.text_type):
			data = data.encode()
		result = evp.DigestUpdate(self, data, len(data))
		if result != 1:
			raise DigestError("Unable to update digest")

	def digest(self, data=None):
		if data is not None:
			self.update(data)
		result_buffer = ctypes.create_string_buffer(evp.MAX_MD_SIZE)
		result_length = ctypes.c_uint()
		res_code = evp.DigestFinal_ex(
			self, result_buffer,
			result_length)
		if res_code != 1:
			raise DigestError("Unable to finalize digest")
		self.finalized = True
		result = result_buffer.raw[:result_length.value]
		# override self.digest to return the same result on subsequent
		#  calls
		self.digest = lambda: result
		return result


evp._set_digest_arg_types(DigestType, Digest)



import py.test

from jaraco.crypto import rand


def test_bytes():
	bytes = rand.bytes(100)
	assert len(bytes) == 100
	assert bytes != rand.bytes(100)


def test_pseudo_bytes():
	bytes = rand.pseudo_bytes(100)
	assert len(bytes) == 100
	assert bytes != rand.pseudo_bytes(100)


def test_seed():
	py.test.skip("This fails, why?")
	seed = 'bunch of bytes' * 1000
	rand.cleanup()
	rand.seed(seed)
	bytes1 = rand.pseudo_bytes(100)
	rand.cleanup()
	rand.seed(seed)
	bytes2 = rand.pseudo_bytes(100)
	assert bytes1 == bytes2



import binascii

import py.test

from jaraco.crypto import digest


def test_load_valid_digest_type_by_name():
	digest.DigestType.from_name('SHA256')


def test_load_invalid_digest_type_by_name():
	# dne is Does Not Exist
	py.test.raises(digest.DigestError, digest.DigestType.from_name, 'sha-dne')


def test_digest():
	digest_type = digest.DigestType.from_name('SHA512')
	sha512 = digest.Digest(digest_type)
	sha512.update("test")
	assert not sha512.finalized
	digest_ = sha512.digest()
	digest_str = binascii.hexlify(digest_).decode('ascii')
	assert len(digest_) == 64
	assert digest_str == (
		"ee26b0dd4af7e749aa1a8ee3c10ae992"
		"3f618980772e473f8819a5d4940e0db2"
		"7ac185f8a0e1d5f84f88bc887fd67b14"
		"3732c304cc5fa9ad8e6f57f50028a8ff")



import ctypes
import os
import platform

from six.moves import filter


def find_library(lib_name):
	"""
	Given a name like libeay32, find the best match.
	"""
	# todo, allow the target environment to customize this behavior
	roots = [
		'c:\\Program Files\\OpenSSL\\',
		'/usr/local/opt/openssl/lib/',
		'/lib/x86_64-linux-gnu/',
	]
	ext = (
		'.dll' if platform.system() == 'Windows' else
		'.dylib' if platform.system() == 'Darwin' else
		'.so.1.0.0' if platform.system() == 'Linux' else
		'.so'
	)
	filename = lib_name + ext
	found = next(_find_file(filename, roots), None)
	return found and ctypes.cdll.LoadLibrary(found)


def _find_file(filename, roots):
	candidates = (
		os.path.join(root, filename)
		for root in roots
	)
	return filter(os.path.exists, candidates)



import platform

import six


collect_ignore = [
	'certutil.py',
]


if platform.system() != 'Windows':
	collect_ignore.extend([
		'jaraco/crypto/cert.py',
	])


if six.PY3:
	collect_ignore.extend([
		'jaraco/crypto/blowfish.py',
	])




from win32com.client import gencache, Dispatch
import win32com.client

clsid = '884e2049-217d-11da-b2a4-000e7bbb2b09'
gencache.EnsureModule('{%s}' % clsid, 0, 1, 0)

class_factory = Dispatch("X509Enrollment.CX509EnrollmentWebClassFactory")
enrollment = class_factory.CreateObject("X509Enrollment.CX509Enrollment")
pkcs10_request = class_factory.CreateObject("X509Enrollment.CX509CertificateRequestPkcs10")
private_key = class_factory.CreateObject("X509Enrollment.CX509privateKey")
distinguished_name = class_factory.CreateObject("X509Enrollment.CX500DistinguishedName")

def generate_request( ):
	generate_key( length = 2048 )
	
	CONTEXT_USER = 1
	CONEXT_MACHINE = 2
	XCN_CERT_NAME_STR_NONE = 0
	
	hresult = pkcs10_request.InitializeFromPrivateKey( CONTEXT_USER, private_key, "" )
	name = 'CN=StartCom Free Certificate Member'
	distinguished_name.Encode( name, XCN_CERT_NAME_STR_NONE )
	pkcs10_request.Subject = distinguished_name

def generate_key( length ):
	key = private_key

	XCN_NCRYPT_ALLOW_EXPORT_FLAG = 1
	XCN_NCRYPT_UI_NO_PROTECTION_FLAG = 0
	XCN_NCRYPT_UI_PROTECT_KEY_FLAG = 1
	
	key.ProviderName = "Microsoft Enhanced RSA and AES Cryptographic Provider"
	key.ProviderType = "24"
	key.KeySpec = "1"
	key.KeyProtection = XCN_NCRYPT_UI_NO_PROTECTION_FLAG
	key.ExportPolicy = XCN_NCRYPT_ALLOW_EXPORT_FLAG
	key.Length = length

generate_request()
enrollment.InitializeFromRequest( pkcs10_request )
print 'Request'
print enrollment.createRequest(1)


import itertools
from ctypes import (
	c_int, c_ulong, c_void_p, c_char_p, POINTER, c_uint, c_char,
)

from .support import find_library


get_digestbyname = None
DigestInit = None
DigestInit_ex = None
MD_CTX_init = None
MD_CTX_create = None
DigestUpdate = None
DigestFinal_ex = None
get_cipherbyname = None

MAX_MD_SIZE = 64

_digest_type_fields = [
	('type', c_int),
	('pkey_type', c_int),
	('md_size', c_int),
	('flags', c_ulong),
	('init', c_void_p),
	('update', c_void_p),
	('final', c_void_p),
	('copy', c_void_p),
	('cleanup', c_void_p),
	('sign', c_void_p),
	('verify', c_void_p),
	('required_pkey_type', c_int * 5),
	('block size', c_int),
	('ctx_size', c_int),
	('md_ctrl', c_void_p),
]

_digest_context_fields = [
	('p_type', c_void_p),  # POINTER(DigestType)
	('engine', c_void_p),  # todo, POINTER(ENGINE)
	('flags', c_ulong),
	('md_data', c_void_p),
	('pctx', c_void_p),  # todo, POINTER(EVP_PKEY_CTX)
	('update_func', c_void_p),
]


def _reg(name):
	"""
	Copy the function by the given name from the EVP library into this
	namespace.
	"""
	libname = 'EVP_' + name
	globals()[name] = getattr(lib, libname)


lib = find_library('libeay32') or find_library('libssl')
assert lib, "Couldn't find OpenSSL"

# Define the argtypes and result types for the EVP functions
list(map(
	_reg, 'get_digestbyname DigestInit DigestInit_ex MD_CTX_init '
	'MD_CTX_create DigestUpdate DigestFinal_ex'.split()))


def _set_digest_arg_types(DigestType, Digest):
	get_digestbyname.argtypes = c_char_p,
	get_digestbyname.restype = POINTER(DigestType)
	DigestInit.argtypes = (
		POINTER(Digest), POINTER(DigestType),
	)
	DigestInit_ex.argtypes = lib.EVP_DigestInit.argtypes + (c_void_p,)
	DigestInit_ex.restype = c_int
	MD_CTX_init.argtypes = POINTER(Digest),
	MD_CTX_create.restype = POINTER(Digest)
	DigestUpdate.argtypes = POINTER(Digest), c_char_p, c_int
	DigestUpdate.restype = c_int
	DigestFinal_ex.argtypes = (
		POINTER(Digest),
		c_char_p, POINTER(c_uint),
	)
	DigestFinal_ex.restype = c_int


_reg('get_cipherbyname')
get_cipherbyname.argtypes = c_char_p,

_cipher_fields = [
	('nid', c_int),
	('block_size', c_int),
	('key_len', c_int),
	('iv_len', c_int),
	('flags', c_ulong),
	('init', c_void_p),
	('do_cipher', c_void_p),
	('cleanup', c_void_p),
	('ctx_size', c_int),
	('set_asn1_parameters', c_void_p),
	('get_asn1_parameters', c_void_p),
	('ctrl', c_void_p),
	('app_data', c_void_p),
]

MAX_IV_LENGTH = 16
MAX_BLOCK_LENGTH = 32
MAX_KEY_LENGTH = 32

_cipher_context_fields = [
	('cipher', c_void_p),  # POINTER(CipherType)
	('engine', c_void_p),  # POINTER(ENGINE)
	('encrypt', c_int),
	('buf_len', c_int),
	('oiv', c_char * MAX_IV_LENGTH),
	('iv', c_char * MAX_IV_LENGTH),
	('buf', c_char * MAX_BLOCK_LENGTH),
	('num', c_int),
	('app_data', c_void_p),
	('key_len', c_int),
	('flags', c_ulong),
	('cipher_data', c_void_p),
	('final_used', c_int),
	('block_mask', c_int),
	('final', c_char * MAX_BLOCK_LENGTH),
]

CIPHER_CTX_init = lib.EVP_CIPHER_CTX_init

# EncryptInit_ex = lib.EVP_EncryptInit_ex
# DecryptInit_ex = lib.EVP_DecryptInit_ex
# ...
for ed, method in itertools.product(
	['Encrypt', 'Decrypt', 'Cipher'],
	['Init_ex', 'Update', 'Final_ex'],
):
	local_name = ''.join([ed, method])
	lib_name = ''.join(['EVP_', ed, method])
	func = getattr(lib, lib_name)
	func.restype = c_int
	globals()[local_name] = func

# Initialize the engines
lib.OpenSSL_add_all_digests()
lib.OpenSSL_add_all_ciphers()



import ctypes


class Stack(ctypes.Structure):
	_fields_ = [
		('num', ctypes.c_int),
		('data', ctypes.POINTER(ctypes.c_char_p)),
		('sorted', ctypes.c_int),
		('num_alloc', ctypes.c_int),
		('comp', ctypes.c_void_p),
	]


class CRYPTO_EX_DATA(ctypes.Structure):
	_fields_ = [
		('sk', ctypes.c_void_p),
		('dummy', ctypes.c_int),
	]


class BIO(ctypes.Structure):
	_fields_ = [
		('method', ctypes.c_void_p),
		('callback', ctypes.c_void_p),
		('cb_arg', ctypes.c_char_p),
		('init', ctypes.c_int),
		('shutdown', ctypes.c_int),
		('flags', ctypes.c_int),
		('retry_reason', ctypes.c_int),
		('num', ctypes.c_int),
		('ptr', ctypes.c_void_p),
		('next_bio', ctypes.c_void_p),
		('prev_bio', ctypes.c_void_p),
		('references', ctypes.c_int),
		('num_read', ctypes.c_ulong),
		('num_write', ctypes.c_ulong),
		('ex_data', CRYPTO_EX_DATA),
	]


FILETYPE_PEM = 1
FILETYPE_ASN1 = 2

libeay32 = ctypes.windll.libeay32
libeay32.BN_bn2hex.restype = ctypes.c_char_p


class X509(ctypes.Structure):
	_fields_ = [

	]

	def get_serial_number(self):
		asn1_i = libeay32.X509_get_serialNumber(ctypes.byref(self))
		bignum = libeay32.ASN1_INTEGER_to_BN(asn1_i, None)
		try:
			hex = libeay32.BN_bn2hex(bignum)
			result = int(hex, 16)
		finally:
			libeay32.BN_free(bignum)
		return result


def load_certificate(type, data):
	bio = libeay32.BIO_new_mem_buf(data, len(data))

	try:
		if type == FILETYPE_PEM:
			cert = libeay32.PEM_read_bio_X509(bio, None, None, None)
		else:
			raise ValueError("Unsupported type")
	finally:
		libeay32.BIO_free(bio)
	if not cert:
		raise Exception("Exception loading cert")
	return ctypes.cast(cert, ctypes.POINTER(X509)).contents


if __name__ == '__main__':
	with open('server.pem', 'rb') as certfile:
		cert_data = certfile.read()
	cert = load_certificate(FILETYPE_PEM, cert_data)
	print(cert.get_serial_number())



import ctypes

import six

from .evp import lib


class RandError(Exception):
	pass


lib.RAND_bytes.argtypes = ctypes.c_char_p, ctypes.c_int
lib.RAND_pseudo_bytes.argtypes = ctypes.c_char_p, ctypes.c_int


def bytes(num, check_result=False):
	if num <= 0:
		raise ValueError("num must be > 0")
	bytes = ctypes.create_string_buffer(num)
	result = lib.RAND_bytes(bytes, num)
	if check_result and result == 0:
		msg = "Random Number Generator not seeded sufficiently"
		raise RandError(msg)
	return bytes.raw[:num]


def pseudo_bytes(num):
	if num <= 0:
		raise ValueError("num must be > 0")
	bytes = ctypes.create_string_buffer(num)
	lib.RAND_pseudo_bytes(bytes, num)
	return bytes.raw[:num]


lib.RAND_seed.argtypes = ctypes.c_char_p, ctypes.c_int
lib.RAND_add.argtypes = ctypes.c_char_p, ctypes.c_int, ctypes.c_double


def seed(data, entropy=None):
	if not isinstance(data, six.string_types):
		raise TypeError("data must be a string")
	params = [data, len(data)]
	if entropy:
		func = lib.RAND_add
		params.append(entropy)
	else:
		func = lib.RAND_seed
	func(*params)


status = lib.RAND_status
cleanup = lib.RAND_cleanup



import ctypes

import six

from . import evp

CIPHER_ALGORITHMS = ("DES", "DES-EDE3", "BF", "AES-128", "AES-192", "AES-256")
CIPHER_MODES = ("CBC", "CFB", "OFB", "ECB")


class CipherError(Exception):
	pass


class CipherType(ctypes.Structure):
	_fields_ = evp._cipher_fields

	@classmethod
	def from_name(cls, *cipher_name):
		"""
		Create a CipherType from a cipher name.

		Takes one or two parameters. If one is supplied, it should be
		a dash-separated string of algorithm-mode.
		If two are supplied, they should be the algorithm and mode.
		"""
		cipher_name = '-'.join(cipher_name)
		algorithm, mode = cipher_name.rsplit('-', 1)
		assert algorithm in CIPHER_ALGORITHMS, (
			"Unknown algorithm %(algorithm)s" % vars())
		assert mode in CIPHER_MODES, "Unknown mode %(mode)s" % vars()
		res = evp.get_cipherbyname(cipher_name.encode('ascii'))
		if not res:
			raise CipherError("Unknown cipher: %(cipher_name)s" % vars())
		res.contents.algorithm, res.contents.mode = algorithm, mode
		return res.contents


evp.get_cipherbyname.restype = ctypes.POINTER(CipherType)


class Cipher(ctypes.Structure):
	_fields_ = evp._cipher_context_fields
	finalized = False

	def __init__(self, type, key, iv, encrypt=True):
		key = key.encode('ascii')
		iv = iv.encode('ascii')
		evp.CIPHER_CTX_init(self)
		engine = None
		type = self.interpret_type(type)
		res = evp.CipherInit_ex(self, type, engine, key, iv, encrypt)
		if res == 0:
			raise CipherError("Unable to initialize cipher")
		self.out_data = []

	@staticmethod
	def interpret_type(type):
		if not isinstance(type, CipherType):
			if not hasattr(type, '__iter__'):
				type = [type]
			type = CipherType.from_name(*type)
		return type

	def set_padding(self, padding=True):
		evp.CIPHER_CTX_set_padding(self, padding)

	def update(self, data):
		"""
		From docs:
		EVP_EncryptUpdate() encrypts inl bytes from the
		buffer in and writes the encrypted version to out.
		This function can be called multiple times to
		encrypt successive blocks of data. The amount of
		data written depends on the block alignment of the
		encrypted data: as a result the amount of data
		written may be anything from zero bytes to
		(inl + cipher_block_size - 1) so outl should
		contain sufficient room. The actual number of
		bytes written is placed in outl.
		"""
		if self.finalized:
			raise CipherError("No updates allowed")
		if isinstance(data, six.text_type):
			data = data.encode()
		out = ctypes.create_string_buffer(len(data) + evp.MAX_BLOCK_LENGTH - 1)
		out_len = ctypes.c_int()

		res = evp.CipherUpdate(self, out, out_len, data, len(data))
		if res != 1:
			raise CipherError("Error updating cipher")
		self.out_data.append(out.raw[:out_len.value])

	def finalize(self, data=None):
		if data is not None:
			self.update(data)
		self.finalized = True
		out = ctypes.create_string_buffer(evp.MAX_BLOCK_LENGTH)
		out_len = ctypes.c_int()
		res = evp.CipherFinal_ex(self, out, out_len)
		if not res == 1:
			raise CipherError("Error finalizing cipher")
		self.out_data.append(out.raw[:out_len.value])
		self.finalize = lambda: ''.join(self.out_data)
		return b''.join(self.out_data)


_init_args = (
	ctypes.POINTER(Cipher), ctypes.POINTER(CipherType),
	ctypes.c_void_p,
	ctypes.c_char_p, ctypes.c_char_p,
)
_update_args = (
	ctypes.POINTER(Cipher), ctypes.c_char_p,
	ctypes.POINTER(ctypes.c_int),
	ctypes.c_char_p, ctypes.c_int,
)
_final_args = (
	ctypes.POINTER(Cipher), ctypes.c_char_p,
	ctypes.POINTER(ctypes.c_int),
)
evp.EncryptInit_ex.argtypes = evp.DecryptInit_ex.argtypes \
	= evp.CipherInit_ex.argtypes = _init_args
evp.EncryptUpdate.argtypes = evp.DecryptUpdate.argtypes \
	= evp.CipherUpdate.argtypes = _update_args
evp.EncryptFinal_ex.argtypes = evp.DecryptFinal_ex.argtypes \
	= evp.CipherFinal_ex.argtypes = _final_args
evp.CIPHER_CTX_init.argtypes = ctypes.POINTER(Cipher),
